@@ -38,6 +38,7 @@ BOOL CreateNewLevelFile()
       fputc(0,file);
   }
   fclose(file);
+  chmod(LEVEL_FILE, LEVEL_PERMS);
   return(TRUE);
 }
 
@@ -58,27 +59,21 @@ BOOL CreateNewScoreFile()
     }
   }
   fclose(file);
+  chmod(SCORE_FILE, SCORE_PERMS);
   return(TRUE);
 }
 
 BOOL CreateNewNamesFile()
 {
-  int i;
   FILE *file;
 
   if (!(file=fopen(NAMES_FILE,"w")))
     return(FALSE);
 
-  fprintf(file,"%s\n",NAMES_COOKIE);	/* Formatkennung */
-  for(i=0;i<MAX_PLAYERS;i++)
-  {
-    fprintf(file,"%s \t",EMPTY_LOGIN);	/* Login-Name */
-    fprintf(file,"%s \t",EMPTY_ALIAS);	/* Alias-Name */
-    fprintf(file,"%d \t",0);		/* Active-Flag */
-    fprintf(file,"%d \t",0);		/* Handicap */
-    fprintf(file,"%x \n",DEFAULT_SETUP);/* Setup-Bits */
-  }
+  fputs(NAMES_COOKIE,file);		/* Formatkennung */
   fclose(file);
+
+  chmod(NAMES_FILE, NAMES_PERMS);
   return(TRUE);
 }
 
@@ -119,8 +114,6 @@ void LoadLevel(int level_nr)
     Ur[15][11] = 25;
     leveltime = 100;
   }
-
-  actual_level = level_nr;
 }
 
 void LoadScore(int level_nr)
@@ -162,49 +155,23 @@ void LoadScore(int level_nr)
   }
 }
 
-void LoadNames()
+BOOL ConvertNames()
 {
-  int i;
+  int i,j;
   char cookie[256];
   FILE *file;
+  char *login_name = GetLoginName();
+  struct PlayerInfo playerlist[MAX_PLAYERS];
 
-  if (!(file=fopen(NAMES_FILE,"r")))
+  if (!(file=fopen(NAMES_FILE,"r")))	/* altes Format lesen ... */
   {
-    if (!CreateNewNamesFile())
-    {
-      fprintf(stderr,"%s: cannot create names file '%s'!\n",
-	      progname,NAMES_FILE);
-    }
-    else if (!(file=fopen(NAMES_FILE,"r"))) 
-    {
-      fprintf(stderr,"%s: cannot load names!\n",
-	      progname);
-    }
+    fprintf(stderr,"%s: cannot convert old names file!\n",progname);
+    return(FALSE);
   }
 
-  if (file)
+  fscanf(file,"%s",cookie);
+  if (!strcmp(cookie,NAMES_COOKIE12))	/* Namensdateiformat 1.2 */
   {
-    fscanf(file,"%s",cookie);
-    if (strcmp(cookie,NAMES_COOKIE))	/* lteres Format? */
-    {
-      rewind(file);			/* dann altes Format lesen ... */
-      for(i=0;i<MAX_PLAYERS;i++)
-      {
-	fscanf(file,"%s",playerlist[i].alias_name);
-	if (!strcmp(playerlist[i].alias_name,EMPTY_ALIAS))
-	  strcpy(playerlist[i].login_name,EMPTY_LOGIN);
-	else
-	  strcpy(playerlist[i].login_name,GetLoginName());
-	fseek(file,i*12+11,0);
-	playerlist[i].handicap = fgetc(file);
-	playerlist[i].last_used = FALSE;
-	playerlist[i].setup = 0;
-      }
-      fclose(file);
-      SaveNames();			/* ... und neu wieder schreiben */
-      return;
-    }
-
     for(i=0;i<MAX_PLAYERS;i++)
     {
       fscanf(file,"%s",playerlist[i].login_name);
@@ -215,17 +182,133 @@ void LoadNames()
     }
     fclose(file);
   }
-  else
+  else					/* Namensdateiformat 1.0 ? */
   {
+    rewind(file);
     for(i=0;i<MAX_PLAYERS;i++)
     {
-      strcpy(playerlist[i].login_name,EMPTY_LOGIN);
-      strcpy(playerlist[i].alias_name,EMPTY_ALIAS);
+      fscanf(file,"%s",playerlist[i].alias_name);
+      if (!strcmp(playerlist[i].alias_name,EMPTY_ALIAS))
+	strcpy(playerlist[i].login_name,EMPTY_LOGIN);
+      else
+	strcpy(playerlist[i].login_name,login_name);
+      fseek(file,i*12+11,0);
+      playerlist[i].handicap = fgetc(file);
       playerlist[i].last_used = FALSE;
-      playerlist[i].handicap = 0;
-      playerlist[i].setup = DEFAULT_SETUP;
+      playerlist[i].setup = 0;
+    }
+    fclose(file);
+  }
+
+  CreateNewNamesFile();			/* ... und neu wieder schreiben */
+
+  for(i=0;i<MAX_PLAYERS;i++)
+  {
+    for(j=0;j<MAX_NAMELEN;j++)
+      player.login_name[j] = player.alias_name[j] = 0;
+    strncpy(player.login_name,playerlist[i].login_name,MAX_NAMELEN-1);
+    strncpy(player.alias_name,playerlist[i].alias_name,MAX_NAMELEN-1);
+    player.handicap = playerlist[i].handicap;
+    player.setup = playerlist[i].setup;
+
+    SavePlayerInfo();
+  }
+
+  return(TRUE);
+}
+
+void LoadPlayerInfo()
+{
+  int i;
+  char cookie[256];
+  FILE *file;
+  char *login_name = GetLoginName();
+  struct PlayerInfo default_player;
+
+  for(i=0;i<MAX_NAMELEN;i++)
+    default_player.login_name[i] = default_player.alias_name[i] = 0;
+  strncpy(default_player.login_name,login_name,MAX_NAMELEN-1);
+  strncpy(default_player.alias_name,login_name,MAX_NAMELEN-1);
+  default_player.handicap = 0;
+  default_player.setup = DEFAULT_SETUP;
+
+  if (!(file=fopen(NAMES_FILE,"r")))
+  {
+    if (!CreateNewNamesFile())
+    {
+      fprintf(stderr,"%s: cannot create names file '%s'!\n",
+	      progname,NAMES_FILE);
+    }
+    else if (!(file=fopen(NAMES_FILE,"r"))) 
+    {
+      fprintf(stderr,"%s: cannot load player information!\n",
+	      progname);
     }
   }
+
+  if (file)
+  {
+    fgets(cookie,NAMES_COOKIE_LEN,file);
+    if (strcmp(cookie,NAMES_COOKIE))	/* lteres Format? */
+    {
+      fclose(file);
+      if (!ConvertNames())
+	file = NULL;
+      else if (!(file=fopen(NAMES_FILE,"r"))) 
+      {
+	fprintf(stderr,"%s: cannot load player information!\n",
+		progname);
+      }
+      else
+	fgets(cookie,NAMES_COOKIE_LEN,file);
+    }
+  }
+
+  if (!file)
+  {
+    player = default_player;
+    level_nr = player.handicap;
+    return;
+  }
+
+  while(1)
+  {
+    for(i=0;i<MAX_NAMELEN;i++)
+      player.login_name[i] = fgetc(file);
+    for(i=0;i<MAX_NAMELEN;i++)
+      player.alias_name[i] = fgetc(file);
+    player.handicap = fgetc(file);
+    player.setup = (fgetc(file)<<8) | fgetc(file);
+
+    if (feof(file))		/* Spieler noch nicht in Liste enthalten */
+    {
+      player = default_player;
+
+      fclose(file);
+      if (!(file=fopen(NAMES_FILE,"a")))
+      {
+	fprintf(stderr,"%s: cannot append new player to names file!\n",
+		progname);
+      }
+      else
+      {
+	for(i=0;i<MAX_NAMELEN;i++)
+	  fputc(player.login_name[i],file);
+	for(i=0;i<MAX_NAMELEN;i++)
+	  fputc(player.alias_name[i],file);
+	fputc(player.handicap,file);
+	fputc(player.setup / 256,file);
+	fputc(player.setup % 256,file);
+      }
+      break;
+    }
+    else			/* prfen, ob Spieler in Liste enthalten */
+      if (!strncmp(player.login_name,login_name,MAX_NAMELEN-1))
+	break;
+  }
+
+  level_nr = player.handicap;
+  fclose(file);
 }
 
 void SaveLevel(int level_nr)
@@ -269,92 +352,79 @@ void SaveScore(int level_nr)
   fclose(file);
 }
 
-void SaveNames()
+void SavePlayerInfo()
 {
   int i;
+  char cookie[256];
   FILE *file;
+  char *login_name = GetLoginName();
+  struct PlayerInfo default_player;
 
-  if (!(file=fopen(NAMES_FILE,"w")))
+  if (!(file=fopen(NAMES_FILE,"r+")))
   {
-    fprintf(stderr,"%s: cannot save names!\n",
+    fprintf(stderr,"%s: cannot save player information!\n",
 	    progname);
     return;
   }
 
-  fprintf(file,"%s\n",NAMES_COOKIE);	/* Formatkennung */
-  for(i=0;i<MAX_PLAYERS;i++)
+  fgets(cookie,NAMES_COOKIE_LEN,file);
+  if (strcmp(cookie,NAMES_COOKIE))	/* ungltiges Format? */
   {
-    fprintf(file,"%s \t",playerlist[i].login_name);
-    fprintf(file,"%s \t",playerlist[i].alias_name);
-    fprintf(file,"%d \t",playerlist[i].last_used);
-    fprintf(file,"%d \t",playerlist[i].handicap);
-    fprintf(file,"%x \n",playerlist[i].setup);
+    fprintf(stderr,"%s: wrong format of names file!\n",progname);
+    fclose(file);
+    return;
   }
+
+  while(1)
+  {
+    for(i=0;i<MAX_NAMELEN;i++)
+      default_player.login_name[i] = fgetc(file);
+    for(i=0;i<MAX_NAMELEN;i++)
+      default_player.alias_name[i] = fgetc(file);
+    default_player.handicap = fgetc(file);
+    default_player.setup = (fgetc(file)<<8) | fgetc(file);
+
+    if (feof(file))		/* Spieler noch nicht in Liste enthalten */
+      break;
+    else			/* prfen, ob Spieler in Liste enthalten */
+      if (!strncmp(default_player.login_name,player.login_name,MAX_NAMELEN-1))
+      {
+	fseek(file,-(2*MAX_NAMELEN+1+2),SEEK_CUR);
+	break;
+      }
+  }
+
+  for(i=0;i<MAX_NAMELEN;i++)
+    fputc(player.login_name[i],file);
+  for(i=0;i<MAX_NAMELEN;i++)
+    fputc(player.alias_name[i],file);
+  fputc(player.handicap,file);
+  fputc(player.setup / 256,file);
+  fputc(player.setup % 256,file);
+
   fclose(file);
 }
 
-void GetPlayerName()
-{
-  int i;
-  int player_pos, empty_pos, last_used;
-  char *login_name;
-
-  LoadNames();
-  login_name = GetLoginName();
-  player_pos = empty_pos = last_used = -1;
-
-  for(i=0;i<MAX_PLAYERS;i++)
-  {
-    if (!strcmp(playerlist[i].login_name,login_name) && player_pos<0)
-      player_pos = i;
-    else if (!strcmp(playerlist[i].login_name,EMPTY_LOGIN) && empty_pos<0)
-      empty_pos = i;
-    if (playerlist[i].last_used)
-      last_used = i;
-  }
-
-  if (player_pos<0 && empty_pos<0)	/* Liste voll, Spieler nicht drin */
-    player = &playerlist[0];
-  else if (player_pos>=0)		/* Spieler bereits in der Liste */
-    player = &playerlist[player_pos];
-  else					/* Spieler noch nicht in der Liste */
-  {
-    strncpy(playerlist[empty_pos].login_name,login_name,MAX_NAMELEN);
-    strncpy(playerlist[empty_pos].alias_name,login_name,MAX_NAMELEN);
-    playerlist[empty_pos].handicap = 0;
-    SaveNames();
-    player = &playerlist[empty_pos];
-  }
-
-  if (last_used>=0 && !strcmp(playerlist[last_used].login_name,login_name))
-    player = &playerlist[last_used];
-
-  level = player->handicap;
-}
-
 void GetPlayerConfig()
 {
   if (sound_status==SOUND_OFF)
-    player->setup &= ~SETUP_SOUND;
+    player.setup &= ~SETUP_SOUND;
   if (!sound_loops_allowed)
   {
-    player->setup &= ~SETUP_SOUND_LOOPS;
-    player->setup &= ~SETUP_SOUND_MUSIC;
+    player.setup &= ~SETUP_SOUND_LOOPS;
+    player.setup &= ~SETUP_SOUND_MUSIC;
   }
 
-  sound_on = SETUP_SOUND_ON(player->setup);
-  sound_loops_on = SETUP_SOUND_LOOPS_ON(player->setup);
-  sound_music_on = SETUP_SOUND_MUSIC_ON(player->setup);
-  toons_on = SETUP_TOONS_ON(player->setup);
+  sound_on = SETUP_SOUND_ON(player.setup);
+  sound_loops_on = SETUP_SOUND_LOOPS_ON(player.setup);
+  sound_music_on = SETUP_SOUND_MUSIC_ON(player.setup);
+  toons_on = SETUP_TOONS_ON(player.setup);
 }
 
 void DrawLevel()
 {
   int x,y;
 
-  if (level!=actual_level) 
-    LoadLevel(level);
-
   G[0].Nr=G[1].Nr=BL=LP=PM=OK=CT=DO=0;
 
   ClearWindow();
@@ -454,7 +524,7 @@ void InitGame()
   LW=First.Wk;
   XS=2*Step[LW].x; 
   YS=2*Step[LW].y;
-  DrawTextExt(pix[DB_DOOR],gc,300+glx,gly,int2str(level,2),FS_SMALL,FC_YELLOW);
+  DrawTextExt(pix[DB_DOOR],gc,300+glx,gly,int2str(level_nr,2),FS_SMALL,FC_YELLOW);
   DrawTextExt(pix[DB_DOOR],gc,300+gbx,gby,int2str(BL,3),FS_SMALL,FC_YELLOW);
   DrawTextExt(pix[DB_DOOR],gc,300+gsx,gsy,int2str(SC,4),FS_SMALL,FC_YELLOW);
   OpenDoor(3);
@@ -1605,11 +1675,11 @@ void GameWon()
 	    DX,DY, DXSIZE,DYSIZE, 300,0);
   CloseDoor(3);
 
-  if (level==player->handicap && level<50) 
+  if (level_nr==player.handicap && level_nr<50) 
   { 
-    player->handicap++; 
+    player.handicap++; 
     bumplevel = TRUE;
-    SaveNames(); 
+    SavePlayerInfo(); 
   }
 
   if ((OK=NewHiScore())>=0) 
@@ -1617,13 +1687,13 @@ void GameWon()
     game_status=HALLOFFAME;
     DrawHallOfFame(OK);
     if (bumplevel)
-      level++;
+      level_nr++;
   }
   else
   {
     game_status=MAINMENU;
     if (bumplevel)
-      level++;
+      level_nr++;
     DrawMainMenu();
   }
   BackToFront();
@@ -1633,9 +1703,9 @@ BOOL NewHiScore()
 {
   int OK=-1;
 
-  LoadScore(level);
+  LoadScore(level_nr);
 
-  if (!strcmp(player->alias_name,EMPTY_ALIAS) || SC<highscore[9].Score) 
+  if (!strcmp(player.alias_name,EMPTY_ALIAS) || SC<highscore[9].Score) 
     return(-1);
 
   for(k=0;k<10;k++) 
@@ -1648,7 +1718,7 @@ BOOL NewHiScore()
 
 #ifdef ONE_PER_NAME
 	for(l=k;l<10;l++)
-	  if (!strcmp(player->alias_name,highscore[l].Name))
+	  if (!strcmp(player.alias_name,highscore[l].Name))
 	    m = l;
 	if (m==k)	/* Spieler berschreibt seine alte Position */
 	  goto put_into_list;
@@ -1662,21 +1732,21 @@ BOOL NewHiScore()
       }
 
       put_into_list:
-      sprintf(highscore[k].Name,player->alias_name);
+      sprintf(highscore[k].Name,player.alias_name);
       highscore[k].Score=SC; 
       OK=k;
       break;
     }
 
 #ifdef ONE_PER_NAME
-    else if (!strcmp(player->alias_name,highscore[k].Name))
+    else if (!strcmp(player.alias_name,highscore[k].Name))
       break;	/* Spieler schon mit besserer Punktzahl in der Liste */
 #endif
 
   }
 
   if (OK>=0) 
-    SaveScore(level);
+    SaveScore(level_nr);
 
   return(OK);
 }
